@@ -10,23 +10,52 @@
 #' "short" - default, values with shorten names,
 #' "full" - full English description,
 #' "polish" - original names in the dataset
+#' @param allow_failure logical - whether to proceed or stop on failure. By default set to TRUE (i.e. don't stop on error). For debugging purposes change to FALSE
 #' @param ... other parameters that may be passed to the 'shortening' function that shortens column names
 #' @importFrom XML readHTMLTable
 #' @importFrom utils download.file unzip read.csv
 #' @importFrom data.table fread
 #' @export
-#'
+#' @returns data.frame with historical hydrological data for the daily time interval
 #' @examples \donttest{
 #'   daily = hydro_imgw_daily(year = 2000)
-#'   head(daily)
 #' }
 #'
 
-hydro_imgw_daily = function(year, coords = FALSE, station = NULL, col_names= "short", ...) {
-  #options(RCurlOptions = list(ssl.verifypeer = FALSE)) # required on windows for RCurl
+hydro_imgw_daily = function(year,
+                            coords = FALSE,
+                            station = NULL,
+                            col_names= "short",
+                            allow_failure = TRUE,
+                            ...) {
+  
+  if (allow_failure) {
+    tryCatch(hydro_imgw_daily_bp(year,
+                                 coords,
+                                 station,
+                                 col_names, 
+                                 ...),
+             error = function(e){
+               message(paste("Problems with downloading data.",
+                             "Run function with argument allow_failure = FALSE",
+                             "to see more details"))})
+  } else {
+    hydro_imgw_daily_bp(year,
+                        coords,
+                        station,
+                        col_names, 
+                        ...)
+  }
+}
 
+#' @keywords internal
+#' @noRd
+hydro_imgw_daily_bp = function(year,
+                               coords,
+                               station,
+                               col_names,
+                               ...) {
   translit = check_locale()
-
   base_url = "https://danepubliczne.imgw.pl/data/dane_pomiarowo_obserwacyjne/dane_hydrologiczne/"
   interval = "daily"
   interval_pl = "dobowe"
@@ -35,29 +64,18 @@ hydro_imgw_daily = function(year, coords = FALSE, station = NULL, col_names= "sh
   test_url(link = paste0(base_url, interval_pl, "/"), output = temp)
   a = readLines(temp, warn = FALSE)
 
-  # if (!httr::http_error(paste0(base_url, interval_pl, "/"))) {
-  #   a = getURL(paste0(base_url, interval_pl, "/"),
-  #              ftp.use.epsv = FALSE,
-  #              dirlistonly = TRUE)
-  # } else {
-  #   stop(call. = FALSE,
-  #        paste0("\nDownload failed. ",
-  #               "Check your internet connection or validate this url in your browser: ",
-  #               paste0(base_url, interval_pl, "/"), "\n"))
-  # }
   ind = grep(readHTMLTable(a)[[1]]$Name, pattern = "/")
   catalogs = as.character(readHTMLTable(a)[[1]]$Name[ind])
   catalogs = gsub(x = catalogs, pattern = "/", replacement = "")
   catalogs = catalogs[catalogs %in% as.character(year)]
   if (length(catalogs) == 0) {
-    stop("Selected year(s) is not available in the database.", call. = FALSE)
+    stop("Selected year(s) is/are not available in the database.", call. = FALSE)
   }
   meta = hydro_metadata_imgw(interval)
 
   all_data = vector("list", length = length(catalogs))
   for (i in seq_along(catalogs)) {
     catalog = catalogs[i]
-    # print(i)
     iterator = c("01", "02", "03", "04", "05", "06",
                 "07", "08", "09", "10", "11", "12")
     data = NULL
@@ -77,17 +95,9 @@ hydro_imgw_daily = function(year, coords = FALSE, station = NULL, col_names= "sh
       }
       # extra exception for a current year according to information provided by IMGW-PIB:
       # i.e.:
-      # "Do czasu zakończenia kontroli przepływów z roku hydrologicznego 2020 (do około poczatku sierpnia 2021),
-      # rekordy z  danymi z roku 2020 mają format:
-      #Kod stacji
-      #Nazwa stacji
-      #Nazwa rzeki/jeziora
-      #Rok hydrologiczny
-      #Wskaźnik miesiąca w roku hydrologicznym
-      #Dzień
-      #Stan wody [cm]
-      #Temperatura wody [st. C]
-      #Miesiąc kalendarzowy
+      # "Do czasu zakonczenia kontroli przeplywow rekordy z danymi z roku 2020 maja format:
+      #Kod stacji  #Nazwa stacji  #Nazwa rzeki/jeziora  #Rok hydrologiczny  #Wskaznik miesiaca w roku hydrologicznym
+      #Dzien  #Stan wody [cm]  #Temperatura wody [st. C]    #Miesiac kalendarzowy
       if (ncol(data1) == 9) {
         data1$flow = NA
         data1 = data1[, c(1:7, 10, 8:9)]
@@ -112,9 +122,7 @@ hydro_imgw_daily = function(year, coords = FALSE, station = NULL, col_names= "sh
 
     colnames(data2) = meta[[2]][, 1]
     all_data[[i]] = merge(data, data2,
-                         by = c("Kod stacji", "Nazwa stacji",
-                               "Rok hydrologiczny", "Nazwa rzeki/jeziora",
-                               "Wskaznik miesiaca w roku hydrologicznym", "Dzien"),
+                         by = intersect(colnames(data), colnames(data2)),
                          all.x = TRUE)
   }
 
@@ -147,11 +155,7 @@ hydro_imgw_daily = function(year, coords = FALSE, station = NULL, col_names= "sh
     }
   }
 
-  all_data = all_data[order(all_data$`Nazwa stacji`,
-                            all_data$`Rok hydrologiczny`,
-                            all_data$`Wskaznik miesiaca w roku hydrologicznym`,
-                            all_data$`Dzien`), ]
-  # dodanie opcji  dla skracania kolumn i usuwania duplikatow:
+  all_data = all_data[do.call(order, all_data[grep(x = colnames(all_data), "Nazwa stacji|Rok hydro|w roku hydro|Dzie")]), ]
   all_data = hydro_shortening_imgw(all_data, col_names = col_names, ...)
 
   return(all_data)
