@@ -3,41 +3,55 @@
 #' Downloading daily (meteorological) data from the Synop stations available in the https://www.ogimet.com/ repository.
 #' The data are processed only if temperature or precipitation fields are present.
 #'
-#' @param date start and finish of date (e.g., date = c("2018-05-01","2018-07-01") )
+#' @param date start and finish of date (e.g., date = c("2018-05-01","2018-07-01") ). By default last 30 days.
 #' @param coords add geographical coordinates of the station (logical value TRUE or FALSE)
 #' @param station WMO ID of meteorological station(s). Character or numeric vector
-#' @param hour time for which the daily raport is generated. Set default as hour = 6
+#' @param hour time for which the daily raport is generated. Set default as hour = 6 (i.e. 6 UTC)
+#' @param allow_failure logical - whether to proceed or stop on failure. By default set to TRUE (i.e. don't stop on error). For debugging purposes change to FALSE
 #' @importFrom XML readHTMLTable
 #' @importFrom utils setTxtProgressBar txtProgressBar
 #' 
 #' @export
-#' 
-#' @keywords internal
+#' @returns data.frame with historical meteorological data for the daily summaries
 #'
-#' @examples \donttest{
-#'   ## downloading data for Poznan-Lawica
-#'   # poznan = ogimet_daily(station = 12330, coords = TRUE)
+#' @examples 
+#' \donttest{
+#'   # downloading daily summaries for last 30 days. station: New York - La Guardia
+#'   new_york = ogimet_daily(station = 72503, coords = TRUE)
 #' }
 #'
 
-ogimet_daily = function(date = c(Sys.Date() - 30, Sys.Date()), 
+ogimet_daily = function(date = c(Sys.Date() - 30, Sys.Date()),
                         coords = FALSE, 
-                        station = c(12326, 12330), 
-                        hour = 6) {
+                        station = NA, 
+                        hour = 6, 
+                        allow_failure = TRUE) {
   
-  #options(RCurlOptions = list(ssl.verifypeer = FALSE)) # required on windows for RCurl
+  if (allow_failure) {
+    tryCatch(ogimet_daily_bp(date = date, coords = coords, station = station, hour = hour), 
+             error = function(e){
+               message(paste("Problems with downloading data.",
+                             "Run function with argument allow_failure = FALSE",
+                             "to see more details"))})
+  } else {
+    ogimet_daily_bp(date = date, coords = coords, station = station, hour = hour)
+  }
+}
 
+#' @keywords Internal
+#' @noRd
+ogimet_daily_bp = function(date = date,
+                        coords = coords, 
+                        station = station, 
+                        hour = hour) {
   dates = seq.Date(min(as.Date(date)), max(as.Date(date)), by = "1 month") 
   dates = unique(c(dates, as.Date(max(date))))
 
   # initalizing empty data frame for storing results:
-
   message(
     paste(
-      "Daily raports were generated starting from",
-      hour,
-      "am each day. Use the hour argument to change it",
-      "\n"
+      "Daily raports will be generated for", hour, 
+      "UTC each day. Use the >>hour<< argument to change it \n"
     )
   )
   data_station <-
@@ -64,7 +78,7 @@ ogimet_daily = function(date = c(Sys.Date() - 30, Sys.Date()),
     )
   
   for (station_nr in station) {
-    print(station_nr)
+    message(station_nr)
     
     # adding progress bar if at least 3 iterations are needed
     if (length(dates) * length(station) >= 3) {
@@ -73,21 +87,23 @@ ogimet_daily = function(date = c(Sys.Date() - 30, Sys.Date()),
 
     for (i in length(dates):1) {
       # update progressbar:
-      if (length(dates) >= 3 ) paste(setTxtProgressBar(pb, abs(length(dates)*length(station) - i)), "\n")
+      if (length(dates) >= 3) paste(setTxtProgressBar(pb, abs(length(dates)*length(station) - i)), "\n")
       
       year = format(dates[i], "%Y")
       month = format(dates[i], "%m")
       day = format(dates[i], "%d")
       ndays = day
+
       linkpl2 = paste("https://www.ogimet.com/cgi-bin/gsynres?lang=en&ind=", station_nr, "&ndays=32&ano=", year, "&mes=", month, "&day=", day, "&hora=", hour,"&ord=REV&Send=Send", sep = "")
-      if (month == 1) linkpl2 = paste("https://www.ogimet.com/cgi-bin/gsynres?lang=en&ind=", station_nr, "&ndays=32&ano=", year, "&mes=", month, "&day=", day, "&hora=", hour, "&ord=REV&Send=Send", sep = "")
-      
-      
       temp = tempfile()
       test_url(linkpl2, temp)
-      
-      # run only if downloaded file is valid
-      if (!is.na(file.size(temp)) & (file.size(temp) > 0)) { 
+      if (is.na(file.size(temp)) | (file.size(temp) < 500)) { 
+        message("Problem with downloading data from:", linkpl2, "\n")
+        if (exists("data_station")) {
+          message("Returning results downloaded up to this point:\n")
+          return(data_station)
+        }
+      } else { # run only if downloaded file is valid
         
         a = readHTMLTable(temp, stringsAsFactors = FALSE)
         unlink(temp)
@@ -109,7 +125,8 @@ ogimet_daily = function(date = c(Sys.Date() - 30, Sys.Date()),
           test = b[1:2, ]
           
           if (is.null(test) ) {
-            warning(paste0("Wrong station ID: ", station_nr, " You can check available stations ID at https://ogimet.com/display_stations.php?lang=en&tipo=AND&isyn=&oaci=&nombre=&estado=&Send=Send"))
+            warning(paste0("Wrong station ID: ", station_nr,
+                           " You can check available stations ID at https://ogimet.com/display_stations.php?lang=en&tipo=AND&isyn=&oaci=&nombre=&estado=&Send=Send"))
             return(data_station)
           } 
           
@@ -153,7 +170,7 @@ ogimet_daily = function(date = c(Sys.Date() - 30, Sys.Date()),
               names_col = "Error_column"
             }
 
-          names_col <-
+          names_col =
             gsub("[^A-Za-z0-9]",
                  "",
                  as.character(lapply(names_col, as.character), stringsAsFactors = FALSE))
@@ -162,10 +179,17 @@ ogimet_daily = function(date = c(Sys.Date() - 30, Sys.Date()),
           b = b[-c(1:2), ]
           b["station_ID"] = station_nr
     
-          # adding year to date
-          b$Date = as.character(paste0(b$Date, "/", year))
-          
-          
+          # extra check if date is for December and January simultanously
+          # e.g. "01/02" "01/01" "12/31" "12/30"
+          uniq_mths = sort(unique(unlist(lapply(strsplit(b$Date, "/"), "[[", 1))))
+          if (sum(uniq_mths %in% c("01", "12")) == 2) {
+            mth = unlist(lapply(strsplit(b$Date, "/"), "[[", 1))
+            yr = ifelse(mth == "01", as.numeric(year), as.numeric(year) - 1)
+            b$Date = as.character(paste0(b$Date, "/", yr))
+          } else {
+            b$Date = as.character(paste0(b$Date, "/", year))
+          }
+
           # to avoid gtools::smartbind function or similar from another package..
           if (ncol(data_station) >= ncol(b)) {
             b[setdiff(names(data_station), names(b))] = NA # adding missing columns
@@ -181,9 +205,6 @@ ogimet_daily = function(date = c(Sys.Date() - 30, Sys.Date()),
     
             }
     
-          # cat(paste(year,month,"\n"))
-          # coords można lepiej na samym koncu dodać kolumne
-          # wtedy jak zmienia się lokalizacja na dacie to tutaj tez
           if (coords) {
             coord = a[[1]][2, 1]
             data_station["Lon"] = get_coord_from_string(coord, "Longitude")
@@ -227,15 +248,15 @@ ogimet_daily = function(date = c(Sys.Date() - 30, Sys.Date()),
       ord1 = c(ord1, setdiff(names(data_station), c("station_ID", "Date", "TemperatureCAvg")))
       data_station = data_station[, ord1]
     }
-    # setdiff(names(df), c("station_ID", "Date", "TC"))
     
     # date to as.Date()
     data_station$Date = as.Date(as.character(data_station$Date), format = "%m/%d/%Y")
     # clipping to interesting period as we're downloading slightly more than needed:
     data_station = data_station[which(data_station$Date >= as.Date(min(date)) & as.Date(data_station$Date) <= as.Date(max(date))), ]
-
+    
   } # end of checking whether no. of rows > 0 
   
-  
+  # removing duplicates:
+  data_station = data_station[row.names(unique(data_station[, c("station_ID", "Date")])), ]
   return(data_station)
 }
